/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef ___RASP24RECORDER_HPP___
#define ___RASP24RECORDER_HPP___

#include "Recorder.hpp"
#include "librasp-netapi.h"
#include "rasp-netapi-arg.h"


#define AD_RASP24AD_CAPTURE_CHANNELS  (AD_CH0_CAPTURE | \
                                       AD_CH1_CAPTURE | \
                                       AD_CH2_CAPTURE | \
                                       AD_CH3_CAPTURE | \
                                       AD_CH4_CAPTURE | \
                                       AD_CH5_CAPTURE | \
                                       AD_CH6_CAPTURE | \
                                       AD_CH7_CAPTURE | \
                                       AD_CH8_CAPTURE )

#define AD_8CH_MODE_CAPTURE_CHANNELS  (AD_CH0_CAPTURE | \
                                       AD_CH1_CAPTURE | \
                                       AD_CH2_CAPTURE | \
                                       AD_CH3_CAPTURE | \
                                       AD_CH4_CAPTURE | \
                                       AD_CH5_CAPTURE | \
                                       AD_CH6_CAPTURE | \
                                       AD_CH7_CAPTURE )

class RASP24Recorder: public Recorder {

private:
  static std::unique_ptr<RASP24Recorder> instance;
  WS_HANDLE_t* h_ws;
  std::shared_ptr<std::thread> recorderThread;
  int header_period;
  std::vector<float> read_buf;
  int read_buf_size;
  int sample_bytes;
  std::chrono::nanoseconds latency;

public:
  RASP24Recorder() :
      h_ws(0), header_period(0), sample_bytes(4), read_buf_size(0) {
    //cerr << "RASP24Recorder\n";
  }

  virtual ~RASP24Recorder() {
    if (h_ws) {
      WsADStop(h_ws);
      WsHandleClose(h_ws);
    }
  }

  static Recorder* GetInstance() {
    if (!instance) {
      instance.reset(new RASP24Recorder);
    }
    return instance.get();
  }

  void operator()();

  virtual int Initialize(const std::string& device_name, int channel_count, int sampling_rate, size_t buf_size);

  //-----------------------------------------------------//
  // start recording
  //-----------------------------------------------------//
  virtual void Start() {
    std::unique_lock<std::mutex> lk(mutex_buffer);
    int ret = WsADStart(h_ws);
    state = RECORDING;
  }

  //-----------------------------------------------------//
  // pause in recording
  //-----------------------------------------------------//
  virtual void Pause() {
    std::unique_lock<std::mutex> lk(mutex_buffer);
    state = PAUSE;
  }

  //-----------------------------------------------------//
  // stop and finish recording
  //-----------------------------------------------------//
  virtual void Stop() {
    std::unique_lock<std::mutex> lk(mutex_buffer);
    state = STOP;
    WsADStop(h_ws);
  }

  //-----------------------------------------------------//
  // Close
  //-----------------------------------------------------//
  virtual void Close() {
    if (h_ws) {
      WsADStop(h_ws);
      WsHandleClose(h_ws);
      h_ws = 0;
    }
  }
};

#endif
