/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef ___RASPRECORDER_HPP___
#define ___RASPRECORDER_HPP___

#include "Recorder.hpp"

#include <fcntl.h>
#include <sys/ioctl.h>
#include "adc16ch.h"

using namespace std;

class RASPRecorder: public Recorder {
private:
  static RASPRecorder* instance;
  int fd_adc;
  std::shared_ptr<std::thread> recorderThread;
  std::vector<short> read_buf;
  int read_buf_size;

public:
  RASPRecorder() :
      fd_adc(0), read_buf_size(128) {
    //cerr << "RASPRecorder\n";
  }

  static Recorder* GetInstance() {
    if (!instance) {
      instance = new RASPRecorder;
    }
    return instance;
  }

  void operator()();
  virtual int Initialize(const std::string& device_name, int channel_count, int sampling_rate, size_t buf_size);

  // start recording
  virtual void Start() {
    std::unique_lock<std::mutex> lk(mutex_buffer);
    state = RECORDING;
    ioctl(fd_adc, FIO_ADC_CONTROL, ADC_START);
  }

  // pause in recording
  virtual void Pause() {
    std::unique_lock<std::mutex> lk(mutex_buffer);
    state = PAUSE;
  }

  // stop and finish recording
  virtual void Stop() {
    std::unique_lock<std::mutex> lk(mutex_buffer);
    state = STOP;
    ioctl(fd_adc, FIO_ADC_CONTROL, ADC_STOP);
  }

  virtual void Close() {
    close(fd_adc);
  }
};

#endif
